from __future__ import unicode_literals

from importlib import import_module

from django.conf import settings  # noqa
from django.core.exceptions import ImproperlyConfigured

from appconf import AppConf


def load_path_attr(path):
    i = path.rfind(".")
    module, attr = path[:i], path[i + 1:]
    try:
        mod = import_module(module)
    except ImportError as e:
        raise ImproperlyConfigured("Error importing %s: '%s'" % (module, e))
    try:
        attr = getattr(mod, attr)
    except AttributeError:
        raise ImproperlyConfigured("Module '%s' does not define a '%s'" % (module, attr))
    return attr


class PinaxPagesAppConf(AppConf):

    HOOKSET = "pinax.submissions.hooks.DefaultHookSet"
    MARKUP_RENDERER = "markdown.markdown"
    FORMS = {}

    def configure_markup_renderer(self, value):
        return load_path_attr(value)

    def configure_hookset(self, value):
        return load_path_attr(value)()

    def configure_forms(self, value):
        forms = {}
        for k, v in value.items():
            forms[k] = load_path_attr(v)
        return forms

    class Meta:
        prefix = "pinax_submissions"



from django.core.management.base import BaseCommand

from ..models import ReviewAssignment, SubmissionBase


class Command(BaseCommand):

    def handle(self, *args, **options):
        for submission in SubmissionBase.objects.filter(cancelled=False):
            print "Creating assignments for {}".format(submission)
            ReviewAssignment.create_assignments(submission)



from django.db import models
from django.db.models import Q
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils.html import strip_tags

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.sites.models import Site


class DefaultHookSet(object):

    def __init__(self):
        from .conf import settings  # if put globally there is a race condition
        self.settings = settings

    def reviewers(self):
        perm = Permission.objects.get(content_type__app_label="reviews", codename="can_review")
        return get_user_model().objects.filter(
            Q(groups__permissions=perm) | Q(user_permissions=perm)
        ).distinct()

    def create_assignments(self, cls, submission, origin):
        reviewers = get_user_model().objects.exclude(
            pk__in=[
                assignment.user_id
                for assignment in cls.objects.filter(
                    submission=submission
                )
            ]
        ).filter(
            groups__name="reviewers",
        ).filter(
            Q(reviewassignment__opted_out=False) | Q(reviewassignment=None)
        ).annotate(
            num_assignments=models.Count("reviewassignment")
        ).order_by(
            "num_assignments", "?",
        )
        num_assigned_reviewers = cls.objects.filter(
            submission=submission,
            opted_out=False
        ).count()
        for reviewer in reviewers[:max(0, cls.NUM_REVIEWERS - num_assigned_reviewers)]:
            cls._default_manager.create(
                submission=submission,
                user=reviewer,
                origin=origin,
            )

    def parse_content(self, content):
        return self.settings.PINAX_SUBMISSIONS_MARKUP_RENDERER(content)

    def send_email(self, to, kind, **kwargs):
        current_site = Site.objects.get_current()
        ctx = {
            "current_site": current_site,
            "STATIC_URL": self.settings.STATIC_URL,
        }
        ctx.update(kwargs.get("context", {}))
        subject = "[%s] %s" % (
            current_site.name,
            render_to_string("pinax/submissions/emails/%s/subject.txt" % kind, ctx).strip()
        )

        message_html = render_to_string("pinax/submission/emails/%s/message.html" % kind, ctx)
        message_plaintext = strip_tags(message_html)

        from_email = self.settings.DEFAULT_FROM_EMAIL

        email = EmailMultiAlternatives(subject, message_plaintext, from_email, to)
        email.attach_alternative(message_html, "text/html")
        email.send()


class HookProxy(object):

    def __getattr__(self, attr):
        from .conf import settings  # if put globally there is a race condition
        return getattr(settings.PINAX_SUBMISSIONS_HOOKSET, attr)


hookset = HookProxy()



from __future__ import unicode_literals

from django import forms

from .models import Review, Comment, SubmissionMessage, SupportingDocument


class SupportingDocumentCreateForm(forms.ModelForm):

    class Meta:
        model = SupportingDocument
        fields = [
            "document",
            "description",
        ]


class ReviewForm(forms.ModelForm):

    class Meta:
        model = Review
        fields = ["comment"]


class ReviewCommentForm(forms.ModelForm):

    class Meta:
        model = Comment
        fields = ["text"]


class SubmitterCommentForm(forms.ModelForm):

    class Meta:
        model = SubmissionMessage
        fields = ["message"]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.core.exceptions import ObjectDoesNotExist
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from model_utils.managers import InheritanceManager

from .hooks import hookset
from .utils import uuid_filename


@python_2_unicode_compatible
class SubmissionKind(models.Model):
    """
    e.g. talk vs panel vs tutorial vs poster
    """
    name = models.CharField(_("Name"), max_length=100)
    slug = models.SlugField(verbose_name=_("Slug"))

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class SubmissionBase(models.Model):

    kind = models.ForeignKey(SubmissionKind, verbose_name=_("Kind"))
    submitter = models.ForeignKey(settings.AUTH_USER_MODEL)
    submitted = models.DateTimeField(
        default=timezone.now,
        editable=False,
        verbose_name=_("Submitted")
    )
    cancelled = models.BooleanField(default=False, verbose_name=_("Cancelled"))

    objects = InheritanceManager()

    def cancel(self):
        self.cancelled = True
        self.save()

    def update_result(self, result):
        if result == "accept":
            self.accept()
        elif result == "reject":
            self.reject()
        elif result == "undecide":
            self.undecide()
        elif result == "standby":
            self.standby()

    def accepted(self):
        self.result.status = "accepted"
        self.result.save()

    def rejected(self):
        self.result.status = "rejected"
        self.result.save()

    def undecide(self):
        self.result.status = "undecided"
        self.result.save()

    def standby(self):
        self.result.status = "standby"
        self.result.save()

    def can_edit(self):
        return True

    @property
    def number(self):
        return str(self.pk).zfill(3)

    @property
    def status(self):
        try:
            return self.result.status
        except ObjectDoesNotExist:
            return _("Undecided")

    def notification_email_context(self):
        return {
            "kind": self.kind.name,
        }

    def __str__(self):
        return "<Submission pk={}, kind={}>".format(self.pk, self.kind)


class SupportingDocument(models.Model):
    submission = models.ForeignKey(SubmissionBase, related_name="supporting_documents", verbose_name=_("Submission"))
    uploaded_by = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("Uploaded by"))
    created_at = models.DateTimeField(default=timezone.now, verbose_name=_("Created at"))
    document = models.FileField(upload_to=uuid_filename, verbose_name=_("Document"))
    description = models.CharField(max_length=140, verbose_name=_("Description"))

    def download_url(self):
        return reverse("submissions_document_download", args=[self.pk, os.path.basename(self.file.name).lower()])


class ReviewAssignment(models.Model):
    AUTO_ASSIGNED_INITIAL = 0
    OPT_IN = 1
    AUTO_ASSIGNED_LATER = 2

    NUM_REVIEWERS = 3

    ORIGIN_CHOICES = [
        (AUTO_ASSIGNED_INITIAL, _("auto-assigned, initial")),
        (OPT_IN, _("opted-in")),
        (AUTO_ASSIGNED_LATER, _("auto-assigned, later")),
    ]

    submission = models.ForeignKey(SubmissionBase, verbose_name=_("Submission"))
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("User"))
    origin = models.IntegerField(choices=ORIGIN_CHOICES, verbose_name=_("Origin"))
    assigned_at = models.DateTimeField(default=timezone.now, verbose_name=_("Assigned at"))
    opted_out = models.BooleanField(default=False, verbose_name=_("Opted out"))

    @classmethod
    def create_assignments(cls, submission, origin=AUTO_ASSIGNED_INITIAL):
        hookset.create_assignments(cls, submission, origin)


class SubmissionMessage(models.Model):
    submission = models.ForeignKey(SubmissionBase, related_name="messages", verbose_name=_("Submission"))
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("User"))
    message = models.TextField(verbose_name=_("Message"))
    message_html = models.TextField(blank=True)
    submitted_at = models.DateTimeField(default=timezone.now, editable=False, verbose_name=_("Submitted at"))

    def save(self, *args, **kwargs):
        self.message_html = hookset.parse_content(self.message)
        return super(SubmissionMessage, self).save(*args, **kwargs)

    class Meta:
        ordering = ["submitted_at"]
        verbose_name = _("submission message")
        verbose_name_plural = _("submission messages")


class Review(models.Model):
    submission = models.ForeignKey(SubmissionBase, related_name="reviews", verbose_name=_("Submission"))
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("User"))
    comment = models.TextField(verbose_name=_("Comment"))
    comment_html = models.TextField(blank=True)
    submitted_at = models.DateTimeField(default=timezone.now, editable=False, verbose_name=_("Submitted at"))

    def save(self, **kwargs):
        self.comment_html = hookset.parse_content(self.comment)
        super(Review, self).save(**kwargs)

    class Meta:
        verbose_name = _("review")
        verbose_name_plural = _("reviews")


class SubmissionResult(models.Model):
    submission = models.OneToOneField(SubmissionBase, related_name="result", verbose_name=_("Submission"))
    accepted = models.NullBooleanField(choices=[
        (True, "accepted"),
        (False, "rejected"),
        (None, "undecided"),
    ], default=None, verbose_name=_("Accepted"))
    status = models.CharField(max_length=20, choices=[
        ("accepted", _("accepted")),
        ("rejected", _("rejected")),
        ("undecided", _("undecided")),
        ("standby", _("standby")),
    ], default="undecided", verbose_name=_("Status"))

    @property
    def accepted(self):
        return self.status == "accepted"

    @property
    def comment_count(self):
        self.submssion.reviews.count()

    class Meta:
        verbose_name = _("submission result")
        verbose_name_plural = _("submission results")


class Comment(models.Model):
    submission = models.ForeignKey(SubmissionBase, related_name="comments", verbose_name=_("Submission"))
    commenter = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("Commenter"))
    text = models.TextField(verbose_name=_("Text"))
    text_html = models.TextField(blank=True)

    # Or perhaps more accurately, can the user see this comment.
    public = models.BooleanField(choices=[(True, _("public")), (False, _("private"))], default=False, verbose_name=_("Public"))
    commented_at = models.DateTimeField(default=timezone.now, verbose_name=_("Commented at"))

    class Meta:
        verbose_name = _("comment")
        verbose_name_plural = _("comments")

    def save(self, *args, **kwargs):
        self.comment_html = hookset.parse_content(self.comment)
        return super(Comment, self).save(*args, **kwargs)


class NotificationTemplate(models.Model):
    label = models.CharField(max_length=100, verbose_name=_("Label"))
    from_address = models.EmailField(verbose_name=_("From address"))
    subject = models.CharField(max_length=100, verbose_name=_("Subject"))
    body = models.TextField(verbose_name=_("Body"))

    class Meta:
        verbose_name = _("notification template")
        verbose_name_plural = _("notification templates")


class ResultNotification(models.Model):
    submission = models.ForeignKey(SubmissionBase, related_name="notifications", verbose_name=_("Submission"))
    template = models.ForeignKey(NotificationTemplate, null=True, blank=True, on_delete=models.SET_NULL, verbose_name=_("Template"))
    timestamp = models.DateTimeField(default=timezone.now, verbose_name=_("Timestamp"))
    to_address = models.EmailField(verbose_name=_("To address"))
    from_address = models.EmailField(verbose_name=_("From address"))
    subject = models.CharField(max_length=100, verbose_name=_("Subject"))
    body = models.TextField(verbose_name=_("Body"))

    def email_args(self):
        return (self.subject, self.body, self.from_address, self.recipients())



from __future__ import unicode_literals

from django.core.mail import send_mass_mail
from django.db.models import Q
from django.http import Http404, HttpResponse, HttpResponseForbidden, HttpResponseBadRequest, HttpResponseNotAllowed
from django.shortcuts import render, redirect, get_object_or_404
from django.template import Context, Template
from django.views import static
from django.views.decorators.http import require_POST

from django.contrib import messages
from django.contrib.auth.models import User

from django.utils.translation import ugettext_lazy as _

from account.decorators import login_required

from .conf import settings
from .forms import (
    ReviewForm,
    SubmitterCommentForm,
    SupportingDocumentCreateForm
)
from .hooks import hookset
from .models import (
    NotificationTemplate,
    ResultNotification,
    Review,
    ReviewAssignment,
    SubmissionBase,
    SubmissionKind,
    SubmissionMessage,
    SubmissionResult,
    SupportingDocument
)


# ******* symposion.proposals.views ******


def submission_submit(request):
    if not request.user.is_authenticated():
        messages.info(request, _("To submit a submission, please "
                                 "<a href='{0}'>log in</a> and create a speaker profile "
                                 "via the dashboard.".format(settings.LOGIN_URL)))
        return redirect("home")  # @@@ unauth'd speaker info page?
    return render(request, "pinax/submissions/submission_submit.html", {
        "kinds": SubmissionKind.objects.all(),
    })


def submission_submit_kind(request, kind_slug):

    kind = get_object_or_404(SubmissionKind, slug=kind_slug)

    if not request.user.is_authenticated():
        return redirect("home")  # @@@ unauth'd speaker info page?

    SubmissionForm = settings.PINAX_SUBMISSIONS_FORMS[kind_slug]

    if request.method == "POST":
        form = SubmissionForm(request.POST)
        if form.is_valid():
            submission = form.save(commit=False)
            submission.submitter = request.user
            submission.kind = kind
            submission.save()
            form.save_m2m()
            messages.success(request, _("Submission submitted."))
            return redirect("dashboard")
    else:
        form = SubmissionForm()

    return render(request, "pinax/submissions/submission_submit_kind.html", {
        "kind": kind,
        "proposal_form": form,
    })


@login_required
def submission_edit(request, pk):
    submission = get_object_or_404(SubmissionBase, pk=pk)
    submission = SubmissionBase.objects.get_subclass(pk=submission.pk)

    if request.user != submission.submitter:
        raise Http404()

    if not submission.can_edit():
        ctx = {
            "title": "Submission editing closed",
            "body": "Submission editing is closed for this session type."
        }
        return render(request, "pinax/submissions/submission_error.html", ctx)

    FormClass = settings.PINAX_SUBMISSIONS_FORMS[submission.kind.slug]

    if request.method == "POST":
        form = FormClass(request.POST, instance=submission)
        if form.is_valid():
            form.save()
            if hasattr(submission, "reviews"):
                users = User.objects.filter(
                    Q(review__submission=submission) |
                    Q(submissionmessage__submission=submission)
                )
                users = users.exclude(pk=request.user.pk).distinct()
                for user in users:
                    ctx = {
                        "user": request.user,
                        "submission": submission,
                    }
                    hookset.send_email(
                        [user.email],
                        "submission_updated",
                        context=ctx
                    )
            messages.success(request, "Submission updated.")
            return redirect("submission_detail", submission.pk)
    else:
        form = FormClass(instance=submission)

    return render(request, "pinax/submissions/submission_edit.html", {
        "submission": submission,
        "form": form,
    })


@login_required
def submission_detail(request, pk):
    submission = get_object_or_404(SubmissionBase, pk=pk, submitter=request.user)
    submission = SubmissionBase.objects.get_subclass(pk=submission.pk)

    message_form = SubmitterCommentForm()
    if request.method == "POST":
        message_form = SubmitterCommentForm(request.POST)
        if message_form.is_valid():
            message = message_form.save(commit=False)
            message.user = request.user
            message.submission = submission
            message.save()

            reviewers = User.objects.filter(
                id__in=SubmissionMessage.objects.filter(
                    submission=submission
                ).exclude(
                    user=request.user
                ).distinct().values_list("user", flat=True)
            )

            for reviewer in reviewers:
                ctx = {
                    "submission": submission,
                    "message": message,
                    "reviewer": True,
                }
                hookset.send_email(
                    [reviewer.email],
                    "submission_new_message",
                    context=ctx
                )

            return redirect(request.path)
    else:
        message_form = SubmitterCommentForm()

    return render(request, "pinax/submissions/submission_detail.html", {
        "submission": submission,
        "message_form": message_form
    })


@login_required
def submission_cancel(request, pk):
    submission = get_object_or_404(SubmissionBase, pk=pk, submitter=request.user)
    submission = SubmissionBase.objects.get_subclass(pk=submission.pk)

    if request.method == "POST":
        submission.cancel()
        # @@@ fire off email to submitter and other speakers
        messages.success(request, "Submission has been cancelled")
        return redirect("dashboard")

    return render(request, "pinax/submissions/submission_cancel.html", {
        "submission": submission,
    })


@login_required
def document_create(request, proposal_pk):
    submission = get_object_or_404(SubmissionBase, pk=proposal_pk, submitter=request.user)
    submission = SubmissionBase.objects.get_subclass(pk=submission.pk)

    if submission.cancelled:
        return HttpResponseForbidden()

    if request.method == "POST":
        form = SupportingDocumentCreateForm(request.POST, request.FILES)
        if form.is_valid():
            document = form.save(commit=False)
            document.submission = submission
            document.uploaded_by = request.user
            document.save()
            return redirect("submission_detail", submission.pk)
    else:
        form = SupportingDocumentCreateForm()

    return render(request, "pinax/submissions/document_create.html", {
        "submission": submission,
        "form": form,
    })


@login_required
def document_download(request, pk, *args):
    document = get_object_or_404(SupportingDocument, pk=pk)
    if getattr(settings, "USE_X_ACCEL_REDIRECT", False):
        response = HttpResponse()
        response["X-Accel-Redirect"] = document.file.url
        # delete content-type to allow Gondor to determine the filetype and
        # we definitely don't want Django's crappy default :-)
        del response["content-type"]
    else:
        response = static.serve(request, document.file.name, document_root=settings.MEDIA_ROOT)
    return response


@login_required
def document_delete(request, pk):
    document = get_object_or_404(SupportingDocument, pk=pk, uploaded_by=request.user)
    if request.method == "POST":
        document.delete()
    return redirect("submission_detail", document.submission.pk)


# ***** symposion.reviews.views *******


def access_not_permitted(request):
    return render(request, "pinax/submissions/access_not_permitted.html")


def submissions_generator(request, queryset, user_pk=None):

    for obj in queryset:
        SubmissionResult.objects.get_or_create(submission=obj)
        lookup_params = dict(submission=obj)
        if user_pk:
            lookup_params["user__pk"] = user_pk
        else:
            lookup_params["user"] = request.user
        yield obj


# Returns a list of all proposals, proposals reviewed by the user, or the proposals the user has
# yet to review depending on the link user clicks in dashboard
@login_required
def review(request, assigned=False, reviewed="all"):

    if not request.user.has_perm("reviews.can_review_submissions"):
        return access_not_permitted(request)

    queryset = SubmissionBase.objects.all()

    if assigned:
        assignments = ReviewAssignment.objects.filter(
            user=request.user
        ).values_list("proposal__id")
        queryset = queryset.filter(id__in=assignments)

    # passing reviewed in from reviews.urls and out to review_list for
    # appropriate template header rendering
    if reviewed == "all":
        queryset = queryset.select_related("result").select_subclasses()
        reviewed = "all_reviews"
    elif reviewed == "reviewed":
        queryset = queryset.filter(reviews__user=request.user)
        reviewed = "user_reviewed"
    else:
        queryset = queryset.exclude(reviews__user=request.user).exclude(submitter=request.user)
        reviewed = "user_not_reviewed"

    submissions = submissions_generator(request, queryset)

    ctx = {
        "submissions": submissions,
        "reviewed": reviewed,
    }

    return render(request, "pinax/submissions/review_list.html", ctx)


@login_required
def review_list(request, user_pk):

    # if they're not a reviewer admin and they aren't the person whose
    # review list is being asked for, don't let them in
    if not request.user.has_perm("reviews.can_manage"):
        if not request.user.pk == user_pk:
            return access_not_permitted(request)

    queryset = SubmissionBase.objects.select_related("result")
    reviewed = Review.objects.filter(user__pk=user_pk).values_list("submission", flat=True)
    queryset = queryset.filter(pk__in=reviewed)
    submissions = queryset.order_by("submitted")
    submissions = submissions_generator(request, submissions, user_pk=user_pk)

    ctx = {
        "submissions": submissions,
    }
    return render(request, "pinax/submissions/review_list.html", ctx)


@login_required
def review_admin(request):
    if not request.user.has_perm("reviews.can_manage"):
        return access_not_permitted(request)

    ctx = {
        "reviewers": hookset.reviewers(),
    }
    return render(request, "pinax/submissions/review_admin.html", ctx)


@login_required
def review_detail(request, pk):
    submissions = SubmissionBase.objects.select_related("result").select_subclasses()
    submission = get_object_or_404(submissions, pk=pk)

    if not request.user.has_perm("reviews.can_review"):
        return access_not_permitted(request)

    admin = request.user.is_staff

    if request.method == "POST":
        if "message_submit" in request.POST:
            message_form = SubmitterCommentForm(request.POST)
            if message_form.is_valid():
                message = message_form.save(commit=False)
                message.user = request.user
                message.submission = submission
                message.save()
                return redirect(request.path)
            else:
                initial = {}
                review_form = ReviewForm(initial=initial)
        elif "result_submit" in request.POST:
            if admin:
                result = request.POST["result_submit"]
                submission.update_result(result)
            return redirect(request.path)
    else:
        initial = {}
        review_form = ReviewForm(initial=initial)
        message_form = SubmitterCommentForm()

    reviews = Review.objects.filter(submission=submission).order_by("-submitted_at")
    messages = submission.messages.order_by("submitted_at")

    return render(request, "pinax/submissions/review_detail.html", {
        "submission": submission,
        "reviews": reviews,
        "review_messages": messages,
        "review_form": review_form,
        "message_form": message_form
    })


@login_required
@require_POST
def review_delete(request, pk):
    review = get_object_or_404(Review, pk=pk)

    if not request.user.has_perm("reviews.can_manage"):
        return access_not_permitted(request)

    review.delete()

    return redirect("submission_detail", pk=review.submission.pk)


@login_required
def review_assignments(request):
    if not request.user.groups.filter(name="reviewers").exists():
        return access_not_permitted(request)
    assignments = ReviewAssignment.objects.filter(
        user=request.user,
        opted_out=False
    )
    return render(request, "pinax/submissions/review_assignment.html", {
        "assignments": assignments,
    })


@login_required
@require_POST
def review_assignment_opt_out(request, pk):
    review_assignment = get_object_or_404(ReviewAssignment, pk=pk, user=request.user)
    if not review_assignment.opted_out:
        review_assignment.opted_out = True
        review_assignment.save()
        ReviewAssignment.create_assignments(review_assignment.proposal, origin=ReviewAssignment.AUTO_ASSIGNED_LATER)
    return redirect("review_assignments")


@login_required
def result_notification(request, status):
    if not request.user.has_perm("reviews.can_manage"):
        return access_not_permitted(request)

    submissions = SubmissionBase.objects.filter(result__status=status).select_related("result").select_subclasses()
    notification_templates = NotificationTemplate.objects.all()

    ctx = {
        "status": status,
        "submissions": submissions,
        "notification_templates": notification_templates,
    }
    return render(request, "pinax/submissions/result_notification.html", ctx)


@login_required
def result_notification_prepare(request, status):
    if request.method != "POST":
        return HttpResponseNotAllowed(["POST"])

    if not request.user.has_perm("reviews.can_manage"):
        return access_not_permitted(request)

    submission_pks = []
    try:
        for pk in request.POST.getlist("_selected_action"):
            submission_pks.append(int(pk))
    except ValueError:
        return HttpResponseBadRequest()
    submissions = SubmissionBase.objects.filter(
        result__status=status,
    )
    submissions = submissions.filter(pk__in=submission_pks)
    submissions = submissions.select_related("result")
    submissions = submissions.select_subclasses()

    notification_template_pk = request.POST.get("notification_template", "")
    if notification_template_pk:
        notification_template = NotificationTemplate.objects.get(pk=notification_template_pk)
    else:
        notification_template = None

    ctx = {
        "status": status,
        "notification_template": notification_template,
        "submissions": submissions,
        "submission_pks": ",".join([str(pk) for pk in submission_pks]),
    }
    return render(request, "pinax/submissions/result_notification_prepare.html", ctx)


@login_required
def result_notification_send(request, status):
    if request.method != "POST":
        return HttpResponseNotAllowed(["POST"])

    if not request.user.has_perm("reviews.can_manage"):
        return access_not_permitted(request)

    if not all([k in request.POST for k in ["submission_pks", "from_address", "subject", "body"]]):
        return HttpResponseBadRequest()

    try:
        submission_pks = [int(pk) for pk in request.POST["submission_pks"].split(",")]
    except ValueError:
        return HttpResponseBadRequest()

    submissions = SubmissionBase.objects.filter(
        result__status=status,
    )
    submissions = submissions.filter(pk__in=submission_pks)
    submissions = submissions.select_related("result")
    submissions = submissions.select_subclasses()

    notification_template_pk = request.POST.get("notification_template", "")
    if notification_template_pk:
        notification_template = NotificationTemplate.objects.get(pk=notification_template_pk)
    else:
        notification_template = None

    emails = []

    for submission in submissions:
        rn = ResultNotification()
        rn.submission = submission
        rn.template = notification_template
        rn.to_address = submission.submitter.email
        rn.from_address = request.POST["from_address"]
        rn.subject = request.POST["subject"]
        rn.body = Template(request.POST["body"]).render(
            Context({
                "submission": submission.notification_email_context()
            })
        )
        rn.save()
        emails.append(rn.email_args)

    send_mass_mail(emails)

    return redirect("result_notification", status=status)



from django.contrib import admin

from .models import SubmissionKind, NotificationTemplate, SubmissionResult


admin.site.register(
    NotificationTemplate,
    list_display=[
        "label",
        "from_address",
        "subject"
    ]
)
admin.site.register(
    SubmissionResult,
    list_display=["submission", "status", "accepted"]
)
admin.site.register(SubmissionKind)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import django.db.models.deletion
import pinax.submissions.utils
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('text', models.TextField(verbose_name='Text')),
                ('text_html', models.TextField(blank=True)),
                ('public', models.BooleanField(default=False, choices=[(True, 'public'), (False, 'private')], verbose_name='Public')),
                ('commented_at', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Commented at')),
                ('commenter', models.ForeignKey(to=settings.AUTH_USER_MODEL, verbose_name='Commenter')),
            ],
            options={
                'verbose_name': 'comment',
                'verbose_name_plural': 'comments',
            },
        ),
        migrations.CreateModel(
            name='NotificationTemplate',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('label', models.CharField(verbose_name='Label', max_length=100)),
                ('from_address', models.EmailField(verbose_name='From address', max_length=254)),
                ('subject', models.CharField(verbose_name='Subject', max_length=100)),
                ('body', models.TextField(verbose_name='Body')),
            ],
            options={
                'verbose_name': 'notification template',
                'verbose_name_plural': 'notification templates',
            },
        ),
        migrations.CreateModel(
            name='ResultNotification',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('timestamp', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Timestamp')),
                ('to_address', models.EmailField(verbose_name='To address', max_length=254)),
                ('from_address', models.EmailField(verbose_name='From address', max_length=254)),
                ('subject', models.CharField(verbose_name='Subject', max_length=100)),
                ('body', models.TextField(verbose_name='Body')),
            ],
        ),
        migrations.CreateModel(
            name='Review',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('comment', models.TextField(verbose_name='Comment')),
                ('comment_html', models.TextField(blank=True)),
                ('submitted_at', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Submitted at')),
            ],
            options={
                'verbose_name': 'review',
                'verbose_name_plural': 'reviews',
            },
        ),
        migrations.CreateModel(
            name='ReviewAssignment',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('origin', models.IntegerField(choices=[(0, 'auto-assigned, initial'), (1, 'opted-in'), (2, 'auto-assigned, later')], verbose_name='Origin')),
                ('assigned_at', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Assigned at')),
                ('opted_out', models.BooleanField(default=False, verbose_name='Opted out')),
            ],
        ),
        migrations.CreateModel(
            name='SubmissionBase',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('submitted', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Submitted')),
                ('cancelled', models.BooleanField(default=False, verbose_name='Cancelled')),
            ],
        ),
        migrations.CreateModel(
            name='SubmissionKind',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('name', models.CharField(verbose_name='Name', max_length=100)),
                ('slug', models.SlugField(verbose_name='Slug')),
            ],
        ),
        migrations.CreateModel(
            name='SubmissionMessage',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('message', models.TextField(verbose_name='Message')),
                ('message_html', models.TextField(blank=True)),
                ('submitted_at', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Submitted at')),
                ('submission', models.ForeignKey(to='submissions.SubmissionBase', related_name='messages', verbose_name='Submission')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name_plural': 'submission messages',
                'ordering': ['submitted_at'],
                'verbose_name': 'submission message',
            },
        ),
        migrations.CreateModel(
            name='SubmissionResult',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('status', models.CharField(default='undecided', choices=[('accepted', 'accepted'), ('rejected', 'rejected'), ('undecided', 'undecided'), ('standby', 'standby')], verbose_name='Status', max_length=20)),
                ('submission', models.OneToOneField(to='submissions.SubmissionBase', related_name='result', verbose_name='Submission')),
            ],
            options={
                'verbose_name': 'submission result',
                'verbose_name_plural': 'submission results',
            },
        ),
        migrations.CreateModel(
            name='SupportingDocument',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Created at')),
                ('document', models.FileField(verbose_name='Document', upload_to=pinax.submissions.utils.uuid_filename)),
                ('description', models.CharField(verbose_name='Description', max_length=140)),
                ('submission', models.ForeignKey(to='submissions.SubmissionBase', related_name='supporting_documents', verbose_name='Submission')),
                ('uploaded_by', models.ForeignKey(to=settings.AUTH_USER_MODEL, verbose_name='Uploaded by')),
            ],
        ),
        migrations.AddField(
            model_name='submissionbase',
            name='kind',
            field=models.ForeignKey(to='submissions.SubmissionKind', verbose_name='Kind'),
        ),
        migrations.AddField(
            model_name='submissionbase',
            name='submitter',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='reviewassignment',
            name='submission',
            field=models.ForeignKey(to='submissions.SubmissionBase', verbose_name='Submission'),
        ),
        migrations.AddField(
            model_name='reviewassignment',
            name='user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AddField(
            model_name='review',
            name='submission',
            field=models.ForeignKey(to='submissions.SubmissionBase', related_name='reviews', verbose_name='Submission'),
        ),
        migrations.AddField(
            model_name='review',
            name='user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AddField(
            model_name='resultnotification',
            name='submission',
            field=models.ForeignKey(to='submissions.SubmissionBase', related_name='notifications', verbose_name='Submission'),
        ),
        migrations.AddField(
            model_name='resultnotification',
            name='template',
            field=models.ForeignKey(to='submissions.NotificationTemplate', on_delete=django.db.models.deletion.SET_NULL, null=True, blank=True, verbose_name='Template'),
        ),
        migrations.AddField(
            model_name='comment',
            name='submission',
            field=models.ForeignKey(to='submissions.SubmissionBase', related_name='comments', verbose_name='Submission'),
        ),
    ]



from django.conf.urls import patterns, url

from . import views


urlpatterns = patterns(
    "",
    url(r"^submit/$", views.submission_submit, name="submission_submit"),
    url(r"^submit/([\w\-]+)/$", views.submission_submit_kind, name="submission_submit_kind"),
    url(r"^(\d+)/$", views.submission_detail, name="submission_detail"),
    url(r"^(\d+)/edit/$", views.submission_edit, name="submission_edit"),
    url(r"^(\d+)/cancel/$", views.submission_cancel, name="submission_cancel"),
    url(r"^(\d+)/document/create/$", views.document_create, name="submission_document_create"),
    url(r"^document/(\d+)/delete/$", views.document_delete, name="submission_document_delete"),
    url(r"^document/(\d+)/([^/]+)$", views.document_download, name="submission_document_download"),

    url(r"^all/$", views.review, {"reviewed": "all"}, name="review_section"),
    url(r"^reviewed/$", views.review, {"reviewed": "reviewed"}, name="user_reviewed"),
    url(r"^not-reviewed/$", views.review, {"reviewed": "not_reviewed"}, name="user_not_reviewed"),
    url(r"^assignments/$", views.review, {"assigned": True}, name="review_section_assignments"),
    url(r"^list/(?P<user_pk>\d+)/$", views.review_list, name="review_list_user"),
    url(r"^admin/$", views.review_admin, name="review_admin"),
    url(r"^notification/(?P<status>\w+)/$", views.result_notification, name="result_notification"),
    url(r"^notification/(?P<status>\w+)/prepare/$", views.result_notification_prepare, name="result_notification_prepare"),
    url(r"^notification/(?P<status>\w+)/send/$", views.result_notification_send, name="result_notification_send"),

    url(r"^reviews/(?P<pk>\d+)/$", views.review_detail, name="review_detail"),

    url(r"^reviews(?P<pk>\d+)/delete/$", views.review_delete, name="review_delete"),
    url(r"^assignments/$", views.review_assignments, name="review_assignments"),
    url(r"^assignment/(?P<pk>\d+)/opt-out/$", views.review_assignment_opt_out, name="review_assignment_opt_out"),
)



from django.core.management.base import BaseCommand

from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType


class Command(BaseCommand):

    def handle(self, *args, **options):
        ct, created = ContentType.objects.get_or_create(
            model="",
            app_label="reviews",
            defaults={"name": "reviews"}
        )
        for action in ["review", "manage", "review_submissions"]:
            perm, created = Permission.objects.get_or_create(
                codename="can_{}".format(action),
                content_type__pk=ct.id,
                defaults={"name": "Can {}".format(action), "content_type": ct}
            )
            print(perm)
